//
//  NimbleVRViewController.m
//  lens
//
//  Created by Vaugham Hong on 2015-12-14.
//  Copyright © 2015 uForis. All rights reserved.
//

#import "NimbleVRViewController.h"
#import "Application.h"

@interface NimbleVRViewController() <CBDStereoRendererDelegate>{
    student_housing::Application *m_pApplication;
    bool m_buttonStateDidChange;
    NSString *_metaDataLocation;
    NSString *_dataInjection;
}
@end

@implementation NimbleVRViewController
@synthesize glview = m_glview;

- (instancetype)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (!self) {return nil; }

    self.stereoRendererDelegate = self;

    m_buttonStateDidChange = false;

    return self;
}

- (void)viewDidLoad {
    self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;

    [super viewDidLoad];
}
- (void)viewWillUnload {
    [super viewWillUnload];
}

- (void)setupRendererWithView:(GLKView *)glView {
    self.glview = glView;
  
    UIDevice *device = [UIDevice currentDevice];
    NSString *currentDeviceId = [[device identifierForVendor]UUIDString];
  
    CGRect screenBounds = [[UIScreen mainScreen] bounds];
    CGFloat screenScale = [[UIScreen mainScreen] scale];
    CGSize screenSize = CGSizeMake(screenBounds.size.width * screenScale, screenBounds.size.height * screenScale);
    CGFloat width = screenSize.width;
    CGFloat height = screenSize.height;

    // Generate info for our application.
    const char *pResourcePath = [[[NSBundle mainBundle] resourcePath] UTF8String];
    const char *pDocumentsPath = [[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0] UTF8String];
    std::string resourcePath = pResourcePath;
    std::string documentsPath = std::string(pDocumentsPath) + "/uforis";
    std::string cachePath = documentsPath + "/cache";
    std::string deviceId = [currentDeviceId UTF8String];
  
  
    [EAGLContext setCurrentContext:glView.context];
  
    // The application initializes the engine and acts as the interface for updates / rendering.
    m_pApplication = new student_housing::Application();
    nimble::app::ApplicationDesc appDesc;
    appDesc.deviceId = deviceId;
    appDesc.launchUrl = "";
    appDesc.resourcePath = resourcePath;
    appDesc.documentsPath = documentsPath;
    appDesc.cachePath = cachePath;
    appDesc.configPath = "app://documents/data/config.json";
    appDesc.screenSize[0] = static_cast<int>(width > height ? width : height);
    appDesc.screenSize[1] = static_cast<int>(width > height ? height : width);
    appDesc.pView = (__bridge void*)self.glview;
    if(_metaDataLocation) {
      const char *charMetaData = [_metaDataLocation UTF8String];
      appDesc.metaData = charMetaData;
    }
  
    m_pApplication->initialize(appDesc);
  
    if(_dataInjection) {
        m_pApplication->engineDataWasInjected([_dataInjection UTF8String]);
    }

    // Use single-tap gesture
    UITapGestureRecognizer *doubleTapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(didSingleTapView:)];
    doubleTapGesture.numberOfTapsRequired = 1;
    [self.view addGestureRecognizer:doubleTapGesture];
}

- (void)shutdownRendererWithView:(GLKView *)glView {
    m_pApplication->destroy();
    delete m_pApplication;
    m_pApplication = 0;
}

- (void)renderViewDidChangeSize:(CGSize)size {
}

- (void)prepareNewFrameWithHeadRotation:(GLKQuaternion)rotation {
  
    nimble::HmdPoseState state;
    state.position[0] = 0.0f;
    state.position[1] = 0.0f;
    state.position[2] = 0.0f;
#if TARGET_OS_SIMULATOR
    // The simulator seems to produce a default rotation that
    // looks down so we hard code it to look forward instead.
    state.rotation[0] = 0.0;
    state.rotation[1] = 0.0;
    state.rotation[2] = 0.0;
    state.rotation[3] = 1.0;
#elif TARGET_OS_IPHONE
    state.rotation[0] = rotation.q[0];
    state.rotation[1] = rotation.q[1];
    state.rotation[2] = rotation.q[2];
    state.rotation[3] = rotation.q[3];
#endif
    m_pApplication->hmdPoseStateDidChange(state);

    m_pApplication->updateSimulation();

    if(m_buttonStateDidChange == true){
        nimble::TouchState state;
        state.state = 0;
        m_pApplication->touchStateDidChange(state);
        m_buttonStateDidChange = false;
    }
}
- (void)drawEyeWithEye:(CBDEye *)eye {
    int eyeIndex = 0;
    if(eye.type == CBDEyeTypeMonocular){ // mono
        eyeIndex = 0;
    }else if(eye.type == CBDEyeTypeLeft){ // left
        eyeIndex = 0;
    }else if(eye.type == CBDEyeTypeRight){ // right
        eyeIndex = 1;
    }

    GLint viewport[4];
    glGetIntegerv(GL_VIEWPORT, viewport);

		m_pApplication->eyeViewPortDidChange(
      eyeIndex,
      viewport[0], viewport[1],
      viewport[2], viewport[3]);
  
    m_pApplication->renderEye(eyeIndex);
}
- (void)finishFrameWithViewportRect:(CGRect)viewPort {
}
- (void)didSingleTapView:(id)sender {
    nimble::TouchState state;
    state.state = nimble::kTouchStateSingleTap;
    m_pApplication->touchStateDidChange(state);
    m_buttonStateDidChange = true;
}

-(void)setMetaDataLocation:(NSString *) metaDataLocation {
  _metaDataLocation = metaDataLocation;
  
  if(m_pApplication && metaDataLocation) {
    nimble::engine::Context *pContext = m_pApplication->getEngineContext();
    pContext->m_platformContext.metaData = [metaDataLocation UTF8String];
  }
}

-(void)injectData:(NSString *)data {
  _dataInjection = data;
  
  if(m_pApplication) {
    m_pApplication->engineDataWasInjected([data UTF8String]);
  } else {
    //Send an error if the application does not exist yet when the injection happens
  }
}


@end
